@@ -1,28 +1,6 @@
 package FusionInventory::Agent::Task::OcsDeploy;
 use threads;
-our $VERSION = '1.0';
-# TODO
-# TIMEOUT="30" number of retry to do on a download
-# CYCLE_LATENCY="60" time to wait between each different priority processing
-# PERIOD_LENGTH="10" nbr of cylce during a period
-#
-# DONE
-# FRAG_LATENCY="10" time to wait between to frag
-# PERIOD_LATENCY="1" time to wait between to package
-
-#
-# period()
-#  for i in PERIOD_LENGTH
-#    foreach priority
-#      foreach package per priority
-#         ' download each frags
-#         ' sleep()FRAG_LATENCY
-#      - then sleep(CYCLE_LATENCY)
-#    - at the end sleep(PERIOD_LATENCY)
-#
-#
-#
-#
+our $VERSION = '1.0.8';
 
 use strict;
 use warnings;
@@ -30,7 +8,7 @@ use warnings;
 use Carp;
 use XML::Simple;
 use File::Copy;
-use File::Glob;
+use File::Glob ':glob';
 use File::Path;
 use File::stat;
 use Digest::MD5 qw(md5);
@@ -38,7 +16,7 @@ use Digest::MD5 qw(md5);
 use File::Copy::Recursive qw(dirmove);
 use Time::HiRes;
 
-use Cwd;
+use Cwd qw(getcwd realpath);
 
 use FusionInventory::Logger;
 use FusionInventory::Agent::Storage;
@@ -58,8 +36,10 @@ sub main {
             }
         });
 
-    my $data = $storage->restore("FusionInventory::Agent");
-    my $myData = $self->{myData} = $storage->restore(__PACKAGE__);
+    my $data = $storage->restore({
+            module => "FusionInventory::Agent"
+        });
+    my $myData = $self->{myData} = $storage->restore();
 
     my $config = $self->{config} = $data->{config};
     my $target = $self->{'target'} = $data->{'target'};
@@ -88,10 +68,11 @@ sub main {
         $logger->fault('No vardir in $target');
     }
 
-    $self->{downloadBaseDir} = $self->{'target'}->{'vardir'} . '/deploy';
-    $self->{runBaseDir}      = $self->{target}->{vardir} . '/run';
-    $self->{tmpBaseDir}      = $self->{target}->{vardir} . '/tmp';
+    my $vardir = realpath($self->{'target'}->{'vardir'});
 
+    $self->{downloadBaseDir} =  $vardir. '/deploy';
+    $self->{runBaseDir}      = $vardir . '/run';
+    $self->{tmpBaseDir}      = $vardir . '/tmp';
 
     foreach (qw/downloadBaseDir runBaseDir tmpBaseDir/) {
         if ( !-d $self->{$_} && !mkpath( $self->{$_} ) ) {
@@ -180,7 +161,7 @@ sub main {
     }
 
 
-    $storage->save($myData);
+    $storage->save({ data => $myData });
 
     exit(0);
 }
@@ -192,7 +173,55 @@ sub diskIsFull {
 
     my $spaceFree;
     if ($^O =~ /^MSWin/) {
-        $logger->fault("isDiskFull doesn't work on Windows");
+
+        if (!eval ('
+                use Win32::OLE qw(in CP_UTF8);
+                use Win32::OLE::Const;
+
+                Win32::OLE->Option(CP => CP_UTF8);
+
+                1')) {
+            $logger->error("Failed to load Win32::OLE: $@");
+        }
+
+
+        my $letter;
+        if ($self->{downloadBaseDir} !~ /^(\w):./) {
+            $logger->error("Path parse error: ".$self->{downloadBaseDir});
+            return;
+        }
+        $letter = $1.':';
+
+
+        my $WMIServices = Win32::OLE->GetObject(
+            "winmgmts:{impersonationLevel=impersonate,(security)}!//./" );
+
+        if (!$WMIServices) {
+            $logger->error(Win32::OLE->LastError());
+            return;
+        }
+
+        foreach my $properties ( Win32::OLE::in(
+                $WMIServices->InstancesOf('Win32_LogicalDisk'))) {
+
+            next unless lc($properties->{Caption}) eq lc($letter);
+            my $t = $properties->{FreeSpace};
+            if ($t && $t =~ /(\d+)\d{6}$/) {
+                $spaceFree = $1;
+            }
+        }
+    } elsif ($^O =~ /^solaris/i) {
+        my $dfFh;
+        if (open($dfFh, '-|', "df", '-b', $self->{downloadBaseDir})) {
+            foreach(<$dfFh>) {
+                if (/^\S+\s+(\d+)/) {
+                    $spaceFree = int($1/1024);
+                }
+            }
+            close $dfFh
+        } else {
+            $logger->error("Failed to exec df");
+        }
     } else {
         my $dfFh;
         if (open($dfFh, '-|', "df", '-Pm', $self->{downloadBaseDir})) {
@@ -292,7 +321,7 @@ sub extractArchive {
                 "Unknow magic number $magicNumber! "
               . "Sorry I can't extract this archive ( $downloadDir/final ). "
               . "If you think, your archive is valide, please submit a bug on "
-              . "http://launchpad.net/ocsinventory with this message and the "
+              . "http://Forge.FusionInventory.org with this message and the "
               . "archive." );
         return;
     }
@@ -301,7 +330,7 @@ sub extractArchive {
     if ($@) {
         $logger->debug("Archive::Extract not found: $@, will use tar directly.");
 	if ($type->{$magicNumber} eq 'tgz') {
-            system("cd $runDir && gunzip -q < $downloadDir/final | tar xvf -")
+            system("cd \"$runDir\" && gunzip -q < \"$downloadDir/final\" | tar xvf -")
         } else {
             $logger->error("Archive type: `".$type->{$magicNumber}.
                             " not supported. Please install ".
@@ -353,9 +382,9 @@ sub processOrderCmd {
             $self->clean( { orderId => $orderId, purge => 1 } );
             return;
         }
-        foreach ( glob("$runDir/*") ) {
+        foreach ( bsd_glob("$runDir/*") ) {
             if (   ( -d $_ && !dirmove( $_, $order->{PATH} ) )
-                && ( -f $_ && !move( $_, $order->{PATH} ) ) )
+                || ( -f $_ && !move( $_, $order->{PATH} ) ) )
             {
                 $self->reportError( $orderId,
                     "Failed to copy $_ in " . $order->{PATH} . " :$!" );
@@ -414,7 +443,7 @@ sub processOrderCmd {
         }
 
     }
-    $self->setErrorCode('CODE_SUCCESS');
+    $self->setErrorCode('SUCCESS_OK');
     $self->reportError( $orderId, "order processed" );
 
     return 1;
@@ -432,7 +461,7 @@ sub downloadAndConstruct {
     my $orderId = $params->{orderId};
     my $order   = $myData->{byId}->{$orderId};
 
-    my $downloadBaseDir = $target->{vardir} . '/deploy';
+    my $downloadBaseDir = $self->{downloadBaseDir};
     my $downloadDir     = $downloadBaseDir . '/' . $orderId;
     if ( !-d $downloadDir && !mkpath($downloadDir) ) {
         $logger->error("Failed to create $downloadDir");
@@ -494,7 +523,8 @@ sub downloadAndConstruct {
             # Can't find a mirror in my networks with the file, I grab it
             # directly from the main server
             $remoteFile = $baseUrl . '/' . $frag;
-            sleep($fragLatency);
+            #Already slow actually
+            #sleep($fragLatency);
         }
         my $localFile = $downloadDir . '/' . $frag;
 
@@ -703,8 +733,6 @@ sub readProlog {
         ];
     }
 
-    my $downloadBaseDir = $target->{vardir} . '/download';
-
     # The orders are send during the PROLOG. Since the prolog is
     # one of the arg of the check() function. We can process it.
     if (!$prologresp) {
@@ -868,16 +896,16 @@ sub findMirror {
     } elsif ( $^O =~ /^linux/x ) {
         foreach (`ifconfig`) {
             if
-            (/inet\saddr:(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}).*Mask:255.255.255.0$/x) {
+            (/inet\saddr:(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}).*Mask:255.255.\d+.\d+$/x) {
                 push @addresses, $1;
             }
 
         }
     }
     elsif ( $^O =~ /^MSWin/x ) {
-        foreach (`route print -4`) {
+        foreach (`route print`) {
             next unless
-            /^\s+\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\s+255\.255\.255\.0/x;
+            /^\s+\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\s+255\.255\.\d+\.\d+/x;
             if (/(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s+\d+$/x) {
                 push @addresses, $1;
             }
@@ -886,6 +914,7 @@ sub findMirror {
 
     foreach my $ip (@addresses) {
         next if $ip =~ /^127/x; # Ignore 127.x.x.x addresses
+        next if $ip =~ /^169/x; # Ignore 169.x.x.x range too
         if ($ip =~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/x) {
 
             foreach (1..255) {
@@ -936,15 +965,16 @@ sub findMirror {
                         my $rc;
                         my $begin;
                         my $end;
+			$logger->debug("url: $url");
                         eval {
                             local $SIG{ALRM} = sub { die "alarm\n" };
-                            alarm 1;
+                            alarm 5;
                             $begin = Time::HiRes::time();
 
                             $rc = $network->getStore({
                                     source => $url,
                                     target => $tempFile,
-                                    timeout => 1
+                                    timeout => 3
                                 }) or croak;
 
                             alarm 0;
@@ -962,7 +992,7 @@ sub findMirror {
 
                     # https://rt.cpan.org/Public/Bug/Display.html?id=41007
                     # http://www.perlmonks.org/index.pl?node_id=407374
-                    if ( $^O =~ /^MSWin/x ) {
+                    if ( 0 ) {
 
                         if ( @{$self->{findMirrorThreads}} > 1 ) {
                             ($lastValdidIp, $url) = $self->_joinFindMirrorThread();
